import matplotlib.pyplot as plt
import numpy as np

import time
from scipy.optimize import fsolve

import scipy
import pickle



def pulse3(x, xe0, xe1, xe2, xe3):
    x0, e0 = xe0
    x1, e1 = xe1
    x2, e2 = xe2
    x3, e3 = xe3
    if np.isscalar(x):
        if x<x0:
            return e0
        elif x<x1:
            return (-(x-x0)*e1 + (x-x1)*e0)/(x0-x1)
        elif x<x2:
            return (-(x-x1)*e2 + (x-x2)*e1)/(x1-x2)
        elif x<x3:
            return (-(x-x2)*e3 + (x-x3)*e2)/(x2-x3)
        else:
            return e3
    else:   
        y = np.zeros(x.shape)
        
        mask0 = x<x0
        mask01 = np.logical_and(x>=x0,x<x1)
        mask12 = np.logical_and(x>=x1,x<x2)
        mask23 = np.logical_and(x>=x2,x<x3)
        mask3 = x>=x3
        y[mask0] = e0
        y[mask01] = (-(x[mask01]-x0)*e1 + (x[mask01]-x1)*e0)/(x0-x1)
        y[mask12] = (-(x[mask12]-x1)*e2 + (x[mask12]-x2)*e1)/(x1-x2)
        y[mask23] = (-(x[mask23]-x2)*e3 + (x[mask23]-x3)*e2)/(x2-x3)
        y[mask3] = e3

        return y
    

sq2 = np.sqrt(2)
sq3 = np.sqrt(3)

ueV2GHz = (1e-6*1.6e-19/6.62e-34)/1e9
# = Qobj(np.diag([1,0]))
s_z = np.array([[1,0],[0,-1]])
s_x = np.array([[0,1],[1, 0]])
s_y = np.array([[0,-1j],[1j, 0]])
s_0 = np.eye(2)
s0, sx, sy, sz = s_0, s_x, s_y, s_z


def SU2decompose(mat):
    u0 = np.real( np.trace( mat @ s0 )/2  )
    ux = np.imag( np.trace( mat @ sx )/2  )
    uy = np.imag( np.trace( mat @ sy )/2  )
    uz = np.imag( np.trace( mat @ sz )/2  )
    
    U = s0* u0 +  (ux*sx + uy*sy + uz*sz) 
    uvec = np.array([ux, uy, uz])
    # if  i==0:
    #     print(  np.sqrt(np.sum(uvec**2))  )    
    if np.sqrt(np.sum(uvec**2)) <= 1e-14:
        uvec = np.array([0,0,1])
    else:
        uvec = uvec/np.sqrt(np.sum(uvec**2))
    direction = uvec
    rot_angle = 2*np.arccos( u0)
    return direction, rot_angle





t_in = 2.5 
phi_ins = [50]

es =   np.linspace(20, -20, 1001)  
evs = np.zeros((len(phi_ins), len(es),4))
for ii in range(len(phi_ins)):
    phi_in = phi_ins[ii]
    t = t_in
    phi0 = phi_in*np.pi/180
    q3 = 1   *10
    q2 = 1    *10

    Hsxs0 = np.kron(s_x, s_0)
    Hszs0 = np.kron(s_z, s_0)
    H_Ez = np.array([[ q2,   0,              0,                 0],
                        [  0,  -q2,             0,                 0],
                        [  0,   0, q3*np.cos(phi0),  q3*np.sin(phi0)],
                        [  0,   0, q3*np.sin(phi0), -q3*np.cos(phi0)]]) /2
    for i in range(len(es)):
        H4x4 = t*Hsxs0 + H_Ez + Hszs0*es[i]
        ev, evec = np.linalg.eigh(H4x4)
        evs[ii,i,:] = ev.copy()
        
   




from mpl_toolkits.axes_grid1 import Divider, Size

fig = plt.figure( figsize=(10,4))
HSPACE, VSPACE = 0.5, 0.5
h = [ s for _ in range(30) for s in [Size.Fixed(HSPACE), Size.Fixed(HSPACE)]  ]
v = [ s for _ in range(8) for s in [Size.Fixed(VSPACE), Size.Fixed(VSPACE)]  ]

divider = Divider(fig, (0, 0, 1, 1), h, v, aspect=False)
ax11 = fig.add_axes(divider.get_position(),
                   axes_locator=divider.new_locator(nx=1, ny=1, nx1=4, ny1=4))

ax31_up = fig.add_axes(divider.get_position(),
                   axes_locator=divider.new_locator(nx=6, ny=3, nx1=7, ny1=4))

ax31_dn = fig.add_axes(divider.get_position(),
                   axes_locator=divider.new_locator(nx=6, ny=1, nx1=7, ny1=2))

ax51 = fig.add_axes(divider.get_position(),
                   axes_locator=divider.new_locator(nx=9, ny=1, nx1=12, ny1=4))


t_in = 2.5 
phi_ins = [50]

es =   np.linspace(20, -20, 1001)  
evs = np.zeros((len(phi_ins), len(es),4))
for ii in range(len(phi_ins)):
    phi_in = phi_ins[ii]
    t = t_in
    phi0 = phi_in*np.pi/180
    q3 = 1   
    q2 = 1   

    Hsxs0 = np.kron(s_x, s_0)
    Hszs0 = np.kron(s_z, s_0)
    H_Ez = np.array([[ q2,   0,              0,                 0],
                        [  0,  -q2,             0,                 0],
                        [  0,   0, q3*np.cos(phi0),  q3*np.sin(phi0)],
                        [  0,   0, q3*np.sin(phi0), -q3*np.cos(phi0)]]) /2
    for i in range(len(es)):
        H4x4 = t*Hsxs0 + H_Ez + Hszs0*es[i]
        ev, evec = np.linalg.eigh(H4x4)
        evs[ii,i,:] = ev.copy()
        
ax = ax11
ax.plot( es, evs[0,:,0:2], 'k-', linewidth=0.75)
ax.plot( es, evs[0,:,2:4], 'k-', linewidth=0.75)
ax.set_xlim(-20, 20)
ax.set_xticks([-20, 0, 20])








t_in = 2.5 
phi_ins = [50]

es =   np.linspace(20, -20, 1001)  
evs = np.zeros((len(phi_ins), len(es),4))
for ii in range(len(phi_ins)):
    phi_in = phi_ins[ii]
    t = t_in
    phi0 = phi_in*np.pi/180
    q3 = 10   
    q2 = 10    

    Hsxs0 = np.kron(s_x, s_0)
    Hszs0 = np.kron(s_z, s_0)
    H_Ez = np.array([[ q2,   0,              0,                 0],
                        [  0,  -q2,             0,                 0],
                        [  0,   0, q3*np.cos(phi0),  q3*np.sin(phi0)],
                        [  0,   0, q3*np.sin(phi0), -q3*np.cos(phi0)]]) /2
    for i in range(len(es)):
        H4x4 = t*Hsxs0 + H_Ez + Hszs0*es[i]
        ev, evec = np.linalg.eigh(H4x4)
        evs[ii,i,:] = ev.copy()
        
ax = ax51
ax.plot( es, evs[0,:,0:2], 'k-', linewidth=0.75)
ax.plot( es, evs[0,:,2:4], 'k-', linewidth=0.75)
ax.set_xlim(-20, 20)
ax.set_xticks([-20, 0, 20])
# ax.set_ylim(0.057, 0.093)


# filename = 'Fig2-3-bc'
# plt.savefig(filename+'.pdf')


#%% PhysRevResearch.3.013194

m0 = 9.1e-31 # unit: kg
m = 0.057* m0

w0 = 2*np.pi*150*1e9 # unit: Hz
hplanck = 6.62e-34
hbar = hplanck

# x = np.linspace(-75,75, 201)
# x0 = 50 # unit: nm
# a = 50 # unit: nm



a = 300*1e-9 # unit: m
x = np.linspace(-1.5*a, 1.5*a, 201)

detuning = 20e9*hplanck/a

y = ((0.5*m*((w0)**2)*((x**2-a**2)**2)/(4*a*a)) - detuning*x)/(hplanck*1e9)

S = np.exp(- a*a*m*(w0**2)/(hbar*w0) )
gamma = (1-np.sqrt(1-S*S))/S
N = 1/(1 - 2*gamma*S + gamma*gamma)
tc = 0.75 * N* gamma * (hbar*w0*w0/w0 + a*a*m*w0*w0   )/(hplanck*1e9)
print(f'tc = {tc:.3g}')

plt.figure()
plt.plot(x*1e9, y , 'k'  )


# plt.plot(  [a*0.8 *1e9, a*1.15 *1e9] , [w0/(2*np.pi*1e9) - detuning*a*1e-9/hplanck ,]*2  , 'r' )
# plt.plot(  [-a*0.8 *1e9, -a*1.15 *1e9] , [w0/(2*np.pi*1e9) + detuning*a*1e-9/hplanck ,]*2  , 'b' )





xx = np.linspace( -0.5*a, 2.5*a, 201)

y2 = w0/(2*np.pi*1e9) - detuning*a*1e-9/hplanck
y1 = np.exp( -  (xx-a)**2/(2*hbar/(m*w0))   )*200 + y2

plt.fill_between(xx*1e9, y1=y1, y2=y2, color='lightgray')
plt.plot(   xx*1e9,  y1 , 'r', linewidth=1    )
plt.plot(  [xx[0]*1e9, xx[-1]*1e9] , [y2 ,]*2  , 'r', linewidth=1  )



xx = np.linspace( 0.5*a, -2.5*a, 201)


y2 = (w0/(2*np.pi*1e9) + detuning*a*1e-9/hplanck)
y1 = np.exp( -  (xx+a)**2/(2*hbar/(m*w0))   )*200 + y2

plt.fill_between(xx*1e9, y1=y1, y2=y2, color='lightgray')
plt.plot(   xx*1e9,  y1 , 'b', linewidth=1    )
plt.plot(  [xx[0]*1e9, xx[-1]*1e9] , [y2 ,]*2  , 'b', linewidth=1  )


# plt.ylim(min(y), y[len(x)//2]   )



# filename = 'Fig2-3-a'
# plt.savefig(filename+'.pdf')



