# this file is modified from Supp/uneuql_time/bloch_slider_without_qutip_v3_q1x90.py
import matplotlib.pyplot as plt
from matplotlib.widgets import Slider, Button
import numpy as np
import scipy.linalg
sx = np.array([[0,1],[1,0]],dtype=complex)
sy = np.array([[0,-1j],[1j,0]],dtype=complex)
sz = np.array([[1,0],[0,-1]],dtype=complex)
sm = np.array([[0,0],[1,0]],dtype=complex)




#%%
DEG = np.pi/180
fq2 = 1 
fq3     = 7.15275643e-02    # qubit  frequency
theta = 41.517 * DEG       # q3 angle from sigma_z axis (toward sigma_x axis)
phi = 0 * DEG 




# initial state
psi00 = np.array([[1],[1]], dtype=complex) /np.sqrt(2)
# psi00 = np.array([[0],[1]], dtype=complex)  
# psi00 = np.array([[1],[0]], dtype=complex) 

# psi00 = np.array([[1],[1j]], dtype=complex) /np.sqrt(2)

# psi00 = np.array([[np.exp(-0.1*1j)],[np.exp(0.1*1j)]], dtype=complex) /np.sqrt(2)

#####
outputs = [psi00]
tlists = []
hlists = []




dt = 0.01
H2 = (fq2/2) *  sz
dU2 = scipy.linalg.expm(  -2*np.pi*1j* H2 * dt  )
# H3 = (fq3/2) * (np.cos(theta)*sz + np.sin(theta)*np.cos(phi)*sx  + np.sin(theta)*np.sin(phi)* sy)
# dU3 = scipy.linalg.expm(  -2*np.pi*1j* H3 * dt  )

output = np.array(psi00)
  



# equal time
wait_q4_1 = 9.16 +1.94
wait_q1 = 4.744 +1.94
wait_q4_2 = 9.16 +1.94
add_wait_q1 = 1.94

tlist = np.arange(0, wait_q4_1  , dt)
for j in range(len(tlist)):
    output = dU2 @ output
    outputs += [output    ]
tlists = np.hstack((tlists,tlist))  
hlists = hlists + [[np.sin(theta)*np.cos(phi), np.sin(theta)*np.sin(phi), np.cos(theta)]]*len(tlist)


hlists = hlists + [[0,0,1]]*len(tlist)



hlists_plot = hlists[0::10]
tlists_plot = tlists[0::10]
outputs_plot = outputs[0::10]
#####

from my_bloch import Bloch
def expect(M, ss):
    if isinstance(ss, list):
        slist = []
        for ssx in ss:
            s = ssx.reshape(-1)
            slist += [  np.real(np.dot(s.conj(), M @ s))  ]
        return slist
    else:
        s = ss.reshape(-1)
        return np.real(np.dot(s.conj(), M @ s))
def state2vec(s):
    return expect(sx, s), expect(sy, s), expect(sz, s)

vx, vy, vz = expect(sx, psi00), expect(sy, psi00), expect(sz, psi00)
sphere_interact = None

if sphere_interact==None:

    # fig, axs = plt.subplots( figsize=(7,9))
    pltname = 'Interactive plot with slider'
    fig = plt.figure( pltname, figsize = (7,9))
    sphere_interact=Bloch(fig)
    
else:
    sphere_interact.clear()    
colorlist = list(plt.cm.Oranges(np.linspace(0.9,0,301)))
# sphere_interact.vector_color = ['g', colorlist[0]] 
sphere_interact.point_color = colorlist
# sphere_interact.add_vectors(hlists_plot[0])
sphere_interact.add_states( state2vec(psi00)  )

sphere_interact.zlabel = [r'$\left|\uparrow\right>$', r'$\left|\downarrow\right>$' ]
sphere_interact.render()

#####


axcolor = 'lightgoldenrodyellow'
ax_t = plt.axes([0.1, 0.05, 0.8, 0.03], facecolor=axcolor)
s_t = Slider(ax_t, 't (ns)', tlists_plot[0], tlists_plot[-1], valinit=tlists_plot[0], valstep=0.02, color='C1')
s_t.label.set_size(15)


def update(val):
     
    tau = s_t.val
    idx = np.argmin(abs(tlists_plot-tau))    
    sphere_interact.clear()

    # sphere_interact.add_vectors(hlists_plot[idx])
    sphere_interact.add_states(  state2vec(outputs_plot[idx])  )
    
    if idx > 0:
        k = max(idx-300, 0)
        vx, vy, vz = expect(sx, outputs_plot[idx:k:-1]), expect(sy, outputs_plot[idx:k:-1]), expect(sz, outputs_plot[idx:k:-1]) 
        sphere_interact.add_points([vx,vy,vz], 'm')
        # print(vx)
     
    sphere_interact.render()

s_t.on_changed(update)
plt.show()

#%%
def state2vec(s):
    return expect(sx, s), expect(sy, s), expect(sz, s)

sphere_static = None
if sphere_static==None:
    pltname = f'Supp_q1x90_equal'
    fig = plt.figure( pltname, figsize = (7,9))
    sphere_static=Bloch(fig)
else:
    sphere_static.clear()

sphere_static.zlabel = [r'$\left|\uparrow\right>$', r'$\left|\downarrow\right>$' ]
sphere_static.vector_color = ['g']*3

psi00 = np.array([[1],[1]], dtype=complex) /np.sqrt(2)

fq2 = 1
dt = 0.1
H2 = (fq2/2) *  sz
dU2 = scipy.linalg.expm(  -2*np.pi*1j* H2 * dt  )

sphere_static.add_vectors(  state2vec( psi00)  )
sphere_static.add_vectors(  state2vec(dU2 @ psi00)  )
sphere_static.add_vectors(  state2vec(dU2 @ dU2 @ psi00)  )
sphere_static.add_vectors(  state2vec(dU2 @ dU2 @ dU2 @ psi00)  )

sphere_static.render()


filename = 'Fig2-5-a' 
plt.savefig(filename+'.pdf')




#%%
def state2vec(s):
    return expect(sx, s), expect(sy, s), expect(sz, s)

sphere_static = None
if sphere_static==None:
    pltname = f'Supp_q1x90_equal'
    fig = plt.figure( pltname, figsize = (7,9))
    sphere_static=Bloch(fig)
else:
    sphere_static.clear()

sphere_static.zlabel = [r'$\left|\uparrow\right>$', r'$\left|\downarrow\right>$' ]
sphere_static.vector_color = ['g']*3

psi00 = np.array([[1],[1]], dtype=complex) /np.sqrt(2)

fd = 0.9

fq2 = 1
dt = 0.1
H2 = ((fq2-fd)/2) *  sz
dU2 = scipy.linalg.expm(  -2*np.pi*1j* H2 * dt  )

sphere_static.add_vectors(  state2vec( psi00)  )
sphere_static.add_vectors(  state2vec(dU2 @ psi00)  )
sphere_static.add_vectors(  state2vec(dU2 @ dU2 @ psi00)  )
sphere_static.add_vectors(  state2vec(dU2 @ dU2 @ dU2 @ psi00)  )

sphere_static.render()


filename = 'Fig2-5-b'  
plt.savefig(filename+'.pdf')

#%%
def state2vec(s):
    return expect(sx, s), expect(sy, s), expect(sz, s)

sphere_static = None
if sphere_static==None:
    pltname = f'Supp_q1x90_equal'
    fig = plt.figure( pltname, figsize = (7,9))
    sphere_static=Bloch(fig)
else:
    sphere_static.clear()

sphere_static.zlabel = [r'$\left|\uparrow\right>$', r'$\left|\downarrow\right>$' ]
sphere_static.vector_color = ['g']*3

psi00 = np.array([[1],[1]], dtype=complex) /np.sqrt(2)

fd = 1

fq2 = 1
dt = 0.1
H2 = ((fq2-fd)/2) *  sz
dU2 = scipy.linalg.expm(  -2*np.pi*1j* H2 * dt  )

sphere_static.add_vectors(  state2vec( psi00)  )
sphere_static.add_vectors(  state2vec(dU2 @ psi00)  )
sphere_static.add_vectors(  state2vec(dU2 @ dU2 @ psi00)  )
sphere_static.add_vectors(  state2vec(dU2 @ dU2 @ dU2 @ psi00)  )

sphere_static.render()


filename = 'Fig2-5-c'  
plt.savefig(filename+'.pdf')
